"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const minification_1 = require("./minification");
const bootstrapper_1 = require("./bootstrapper");
const clean_html_1 = require("./clean-html");
class AureliaNodeTemplatingEngine {
    get options() {
        return this._options;
    }
    constructor(view, options) {
        this._view = view;
        this._options = minification_1.getOptions(options);
    }
    compile(context) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this
                .setContext(context)
                .bootstrapAndRender();
            const result = minification_1.minify(clean_html_1.CleanHtml.execute(this._host).innerHTML, this._options);
            this.dispose();
            return result;
        });
    }
    setContext(context) {
        this._context = context;
        return this;
    }
    bootstrapAndRender() {
        return __awaiter(this, void 0, void 0, function* () {
            this._aurelia = yield bootstrapper_1.Bootstrapper.start();
            this.appendView();
            yield this.render();
            return this;
        });
    }
    render() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._aurelia.enhance(this._context, this._host);
        });
    }
    appendView() {
        if (this._view instanceof Element) {
            this._host = this._view;
        }
        else {
            this._host = document.createElement('div');
            this._host.innerHTML = this._view;
        }
        document.body.appendChild(this._host);
    }
    dispose() {
        bootstrapper_1.Bootstrapper.dispose();
        if (this._host.parentNode) {
            this._host.parentNode.removeChild(this._host);
        }
    }
}
exports.AureliaNodeTemplatingEngine = AureliaNodeTemplatingEngine;
